@@ -1,11 +1,11 @@
 /******************************************************************************/
-/* The C Clustering Library for cDNA microarray data.
+/* The C Clustering Library.
  * Copyright (C) 2002 Michiel Jan Laurens de Hoon.
  *
  * This library was written at the Laboratory of DNA Information Analysis,
  * Human Genome Center, Institute of Medical Science, University of Tokyo,
  * 4-6-1 Shirokanedai, Minato-ku, Tokyo 108-8639, Japan.
- * Contact: mdehoon@c2b2.columbia.edu
+ * Contact: mdehoon 'AT' gsc.riken.jp
  * 
  * Permission to use, copy, modify, and distribute this software and its
  * documentation with or without modifications and for any purpose and
@@ -27,9 +27,6 @@
  * 
  */
 
-#ifndef C_CLUSTERING_LIB
-#define C_CLUSTERING_LIB
-
 #ifndef CALL 
 # define CALL
 #endif
@@ -50,23 +47,14 @@
 double CALL clusterdistance (int nrows, int ncolumns, double** data, int** mask,
   double weight[], int n1, int n2, int index1[], int index2[], char dist,
   char method, int transpose);
-
-/* Chapter 3 */
-void CALL initran(void);
-
-/* Chapter 4 */
 double** CALL distancematrix (int ngenes, int ndata, double** data,
   int** mask, double* weight, char dist, int transpose);
 
-/* Chapter 5 */
-void CALL randomassign (int nclusters, int ngenes, int clusterid[]);
-void getclustermean(int nclusters, int nrows, int ncolumns,
+/* Chapter 3 */
+int getclustercentroids(int nclusters, int nrows, int ncolumns,
   double** data, int** mask, int clusterid[], double** cdata, int** cmask,
-  int transpose);
-void getclustermedian(int nclusters, int nrows, int ncolumns,
-  double** data, int** mask, int clusterid[], double** cdata, int** cmask,
-  int transpose);
-void getclustermedoid(int nclusters, int nelements, double** distance,
+  int transpose, char method);
+void getclustermedoids(int nclusters, int nelements, double** distance,
   int clusterid[], int centroids[], double errors[]);
 void CALL kcluster (int nclusters, int ngenes, int ndata, double** data,
   int** mask, double weight[], int transpose, int npass, char method, char dist,
@@ -74,19 +62,29 @@ void CALL kcluster (int nclusters, int ngenes, int ndata, double** data,
 void CALL kmedoids (int nclusters, int nelements, double** distance,
   int npass, int clusterid[], double* error, int* ifound);
 
-/* Chapter 6 */
-int CALL treecluster (int nrows, int ncolumns, double** data, int** mask,
-  double weight[], int transpose, char dist, char method,
-  int result[][2], double linkdist[], double** distmatrix);
-void cuttree (int nelements, int tree[][2], int nclusters, int clusterid[]);
+/* Chapter 4 */
+typedef struct {int left; int right; double distance;} Node;
+/*
+ * A Node struct describes a single node in a tree created by hierarchical
+ * clustering. The tree can be represented by an array of n Node structs,
+ * where n is the number of elements minus one. The integers left and right
+ * in each Node struct refer to the two elements or subnodes that are joined
+ * in this node. The original elements are numbered 0..nelements-1, and the
+ * nodes -1..-(nelements-1). For each node, distance contains the distance
+ * between the two subnodes that were joined.
+ */
 
-/* Chapter 7 */
+Node* CALL treecluster (int nrows, int ncolumns, double** data, int** mask,
+  double weight[], int transpose, char dist, char method, double** distmatrix);
+void cuttree (int nelements, Node* tree, int nclusters, int clusterid[]);
+
+/* Chapter 5 */
 void CALL somcluster (int nrows, int ncolumns, double** data, int** mask,
   const double weight[], int transpose, int nxnodes, int nynodes,
   double inittau, int niter, char dist, double*** celldata,
   int clusterid[][2]);
 
-/* Chapter 8 */
+/* Chapter 6 */
 void CALL svd(int m, int n, double** u, double w[], double** v, int* ierr);
 
 /* Utility routines, currently undocumented */
@@ -96,4 +94,3 @@ double CALL median (int n, double x[]);
 
 double* calculate_weights(int nrows, int ncolumns, double** data, int** mask,
   double weights[], int transpose, char dist, double cutoff, double exponent);
-#endif
