@@ -7,6 +7,10 @@ unambiguous_dna_by_name = {}
 unambiguous_dna_by_id = {}
 unambiguous_rna_by_name = {}
 unambiguous_rna_by_id = {}
+generic_by_name = {} # unambiguous DNA or RNA
+generic_by_id = {} # unambiguous DNA or RNA
+ambiguous_generic_by_name = {} # ambiguous DNA or RNA
+ambiguous_generic_by_id = {} # ambiguous DNA or RNA 
 
 # standard IUPAC unambiguous codons
 standard_dna_table = None
@@ -39,6 +43,63 @@ class CodonTable:
         self.start_codons = start_codons
         self.stop_codons = stop_codons
 
+    def __str__(self) :
+        """Returns a simple text representation of the codon table
+
+        e.g.
+        >>> import Bio.Data.CodonTable
+        >>> print Bio.Data.CodonTable.standard_dna_table
+        >>> print Bio.Data.CodonTable.generic_by_id[1]"""
+
+        if self.id :
+            answer = "Table %i" % self.id
+        else :
+            answer = "Table ID unknown"
+        if self.names :
+            answer += " " + ", ".join(filter(None, self.names))
+
+        #Use the main four letters (and the conventional ordering)
+        #even for ambiguous tables
+        letters = self.nucleotide_alphabet.letters
+        if isinstance(self.nucleotide_alphabet, Alphabet.DNAAlphabet) \
+        or (letters is not None and "T" in letters) :
+            letters = "TCAG"
+        else :
+            #Should be either RNA or generic nucleotides,
+            #e.g. Bio.Data.CodonTable.generic_by_id[1]
+            letters = "TCAG"
+
+        #Build the table...
+        answer=answer + "\n\n  |" + "|".join( \
+            ["  %s      " % c2 for c2 in letters] \
+            ) + "|"
+        answer=answer + "\n--+" \
+               + "+".join(["---------" for c2 in letters]) + "+--"
+        for c1 in letters :
+            for c3 in letters :
+                line = c1 + " |"
+                for c2 in letters :
+                    codon = c1+c2+c3
+                    line = line + " %s" % codon
+                    if codon in self.stop_codons :
+                        line = line + " Stop|"
+                    else :
+                        try :
+                            amino = self.forward_table[codon]
+                        except KeyError :
+                            amino = "?"
+                        except TranslationError :
+                            amino = "?"
+                        if codon in self.start_codons :
+                            line = line + " %s(s)|" % amino
+                        else :
+                            line = line + " %s   |" % amino
+                line = line + " " + c3
+                answer = answer + "\n"+ line 
+            answer=answer + "\n--+" \
+                  + "+".join(["---------" for c2 in letters]) + "+--"
+        return answer
+            
 def make_back_table(table, default_stop_codon):
     #  ONLY RETURNS A SINGLE CODON
     # Do the sort so changes in the hash implementation won't affect
@@ -80,15 +141,29 @@ def register_ncbi_table(name, alt_name, id,
                             stop_codons)
     # replace all T's with U's for the RNA tables
     rna_table = {}
+    generic_table = {}
     for codon, val in table.items():
-        rna_table[string.replace(codon, "T", "U")] = val
+        generic_table[codon] = val
+        codon = codon.replace("T", "U")
+        generic_table[codon] = val
+        rna_table[codon] = val
     rna_start_codons = []
+    generic_start_codons = []
     for codon in start_codons:
-        rna_start_codons.append(string.replace(codon, "T", "U"))
+        generic_start_codons.append(codon)
+        codon = codon.replace("T", "U")
+        generic_start_codons.append(codon)
+        rna_start_codons.append(codon)
     rna_stop_codons = []
+    generic_stop_codons = []
     for codon in stop_codons:
-        rna_stop_codons.append(string.replace(codon, "T", "U"))
+        generic_stop_codons.append(codon)
+        codon = codon.replace("T", "U")
+        generic_stop_codons.append(codon)
+        rna_stop_codons.append(codon)
     
+    generic = NCBICodonTable(id, names + [alt_name], generic_table,
+                             generic_start_codons, generic_stop_codons)
     rna = NCBICodonTableRNA(id, names + [alt_name], rna_table,
                             rna_start_codons, rna_stop_codons)
 
@@ -99,6 +174,7 @@ def register_ncbi_table(name, alt_name, id,
 
     unambiguous_dna_by_id[id] = dna
     unambiguous_rna_by_id[id] = rna
+    generic_by_id[id] = generic
 
     if alt_name is not None:
         names.append(alt_name)
@@ -106,6 +182,7 @@ def register_ncbi_table(name, alt_name, id,
     for name in names:
         unambiguous_dna_by_name[name] = dna
         unambiguous_rna_by_name[name] = rna
+        generic_by_name[name] = generic
 
 ### These tables created from the data file
 ###  ftp://ncbi.nlm.nih.gov/entrez/misc/data/gc.prt
@@ -626,5 +703,25 @@ for key, val in unambiguous_rna_by_id.items():
                                      IUPACData.ambiguous_rna_values,
                                      IUPAC.extended_protein,
                                      IUPACData.extended_protein_values)
-del key, val
 
+#The following isn't very elegant, but seems to work nicely.
+_merged_values = dict()
+_merged_values.update(IUPACData.ambiguous_rna_values)
+_merged_values.update(IUPACData.ambiguous_dna_values)
+
+for key, val in generic_by_name.items():
+    ambiguous_generic_by_name[key] = AmbiguousCodonTable(val,
+                                     Alphabet.NucleotideAlphabet(),
+                                     _merged_values,
+                                     IUPAC.extended_protein,
+                                     IUPACData.extended_protein_values)
+
+for key, val in generic_by_id.items():
+    ambiguous_generic_by_id[key] = AmbiguousCodonTable(val,
+                                     Alphabet.NucleotideAlphabet(),
+                                     _merged_values,
+                                     IUPAC.extended_protein,
+                                     IUPACData.extended_protein_values)
+
+del _merged_values
+del key, val
