@@ -6,12 +6,12 @@
 # as part of this package.
 
 """
-Bio.easy: some functions to ease the use of Biopython
+Bio.GFF.easy: some functions to ease the use of Biopython
 """
 
 from __future__ import generators # requires Python 2.2
 
-__version__ = "$Revision: 1.3 $"
+__version__ = "$Revision: 1.8 $"
 # $Source: /home/repository/biopython/biopython/Bio/GFF/easy.py,v $
 
 import copy
@@ -23,7 +23,12 @@ import Bio
 from Bio import GenBank
 from Bio.Data import IUPACData
 from Bio.Seq import Seq
-from Bio.SeqIO.FASTA import FastaReader, FastaWriter
+
+#from Bio.SeqIO.FASTA import FastaReader, FastaWriter
+#The whole of Bio.SeqIO.FASTA has been deprecated, so
+#we'll use the new Bio.SeqIO functions instead:
+from Bio import SeqIO
+
 from Bio import SeqUtils
 
 import GenericTools
@@ -442,18 +447,30 @@ def fasta_single(filename=None, string=None):
     >>> record.id
     'gi|9629360|ref|NP_057850.1|'
     >>> record.description
-    'Gag [Human immunodeficiency virus type 1]'
+    'gi|9629360|ref|NP_057850.1| Gag [Human immunodeficiency virus type 1]'
     >>> record.seq[0:5]
-    Seq('MGARA', Alphabet())
+    Seq('MGARA', SingleLetterAlphabet())
     """
+    #Returns the first record in a fasta file as a SeqRecord,
+    #or None if there are no records in the file.
     if string:
         import cStringIO
-        return FastaReader(cStringIO.StringIO(string)).next()
-    return FastaReader(open_file(filename)).next()
+        handle = cStringIO.StringIO(string)
+    else :
+        handle = open_file(filename)
+    try :
+        record = SeqIO.parse(handle, format="fasta").next()
+    except StopIteration :
+        record = None
+    return record
 
 def fasta_multi(filename=None):
-    reader = FastaReader(open_file(filename))
-    while 1:
+    #Simple way is just:
+    #return SeqIO.parse(open_file(filename), format="fasta")
+    #However, for backwards compatibility make sure we raise
+    #the StopIteration exception rather than returning None.
+    reader = SeqIO.parse(open_file(filename), format="fasta")
+    while True:
         record = reader.next()
         if record is None:
             raise StopIteration
@@ -466,15 +483,14 @@ def fasta_readrecords(filename=None):
     >>> records[0].id
     'test1'
     >>> records[2].seq
-    Seq('AAACACAC', Alphabet())
+    Seq('AAACACAC', SingleLetterAlphabet())
     """
-    records = []
-    for record in fasta_multi(filename):
-        records.append(record)
-    return records
+    return list(SeqIO.parse(open_file(filename), format="fasta"))
 
 def fasta_write(filename, records):
-    FastaWriter(file(filename, "w")).write_records(records)
+    handle = open(filename, "w")
+    SeqIO.write(records, handle, format="fasta")
+    handle.close()
 
 def genbank_single(filename):
     """
@@ -568,75 +584,11 @@ def record_coords(record, start, end, strand=0, upper=0):
     subseq = record_sequence(record)[start:end]
     subseq_str = subseq.tostring()
     subseq_str = subseq_str.upper()
+    subseq = Seq(subseq_str, subseq.alphabet)
     if strand == '-' or strand == 1:
-        return Seq(SeqUtils.antiparallel(subseq_str), subseq.alphabet)
+        return subseq.reverse_complement()
     else:
-        return Seq(subseq_str, subseq.alphabet)
-
-def forward_complement(seq):
-    """
-    returns the complementary sequence (NOT antiparallel)
-
-    stolen from a broken BioPython sequtils.py
-
-    >>> forward_complement(Seq('aaatttc'))
-    Seq('TTTAAAG', Alphabet())
-    >>> forward_complement(Seq('aaauuuc'))
-    Seq('UUUAAAG', Alphabet())
-    """
-    return _complement(seq, reverse=0)
-
-def _forward_complement_list_with_table(table, seq):
-    return [table[x] for x in seq.tostring().upper()]
-
-from Bio.Data.IUPACData import ambiguous_dna_complement
-ambiguous_rna_complement = ambiguous_dna_complement.copy()
-ambiguous_rna_complement["A"] = "U"
-ambiguous_rna_complement["U"] = "A"
-del ambiguous_rna_complement["T"]
-
-def _forward_complement_list(seq):
-    """
-    assumes DNA, then tries RNA
-    """
-    try:
-        return _forward_complement_list_with_table(ambiguous_dna_complement, seq)
-    except KeyError, err:
-        if err[0] == "U":
-            return _forward_complement_list_with_table(ambiguous_rna_complement, seq)
-        raise
-
-def _complement(seq, reverse=0):
-    """
-    forward or reverse complement
-
-    >>> _complement(Seq('aaatttc'))
-    Seq('TTTAAAG', Alphabet())
-    >>> _complement(Seq('aaatttc'), reverse=1)
-    Seq('GAAATTT', Alphabet())
-    """
-    complement_list = _forward_complement_list(seq)
-    if reverse:
-        complement_list.reverse()
-    return Seq(''.join(complement_list), seq.alphabet)
-
-def reverse_complement(seq):
-    """
-    >>> reverse_complement(Seq('aaatttc'))
-    Seq('GAAATTT', Alphabet())
-    """
-    return _complement(seq, reverse=1)
-
-class TempFastaWriter(FastaWriter):
-    def __init__(self, *args, **keywds):
-        self.file = GenericTools.TempFile()
-        FastaWriter.__init__(self, self.file, *args, **keywds)
-
-class TempFastaWriterSingle(TempFastaWriter):
-    def __init__(self, seqrecord, *args, **keywds):
-        TempFastaWriter.__init__(self, *args, **keywds)
-        self.write(seqrecord)
-        self.close()
+        return subseq
 
 def _test(*args, **keywds):
     import doctest, sys
