@@ -1,8 +1,6 @@
-#include <iostream>
 #include <vector>
 #include <algorithm>
 #include <math.h>
-#include <assert.h>
 #include <stdlib.h>
 using namespace std;
 
