@@ -86,8 +86,8 @@ class _MEMEScanner:
     def _scan_header(self, uhandle, consumer):
         try:
             read_and_call_until(uhandle, consumer.noevent, contains = 'MEME version')
-        except SyntaxError:
-            raise SyntaxError, "Improper input file. File should contain a line starting MEME version."
+        except ValueError:
+            raise ValueError, "Improper input file. File should contain a line starting MEME version."
         read_and_call(uhandle, consumer._version, start = 'MEME version')
         read_and_call_until(uhandle, consumer.noevent, start = 'TRAINING SET')
         read_and_call(uhandle, consumer.noevent, start = 'TRAINING SET')
@@ -499,8 +499,8 @@ class _MASTScanner:
     def _scan_header (self, uhandle, consumer):
         try:
             read_and_call_until(uhandle, consumer.noevent, contains = "MAST version")
-        except SyntaxError:
-            raise SyntaxError, "Improper input file. Does not begin with a line with 'MAST version'"
+        except ValueError:
+            raise ValueError, "Improper input file. Does not begin with a line with 'MAST version'"
         read_and_call(uhandle, consumer._version, contains = 'MAST version')
         read_and_call_until(uhandle, consumer.noevent, start = 'DATABASE AND MOTIFS')
         read_and_call(uhandle, consumer.noevent, start = 'DATABASE')
