@@ -1,9 +1,6 @@
 # Copyright 1999 by Jeffrey Chang.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
-
 # Copyright 2000 by Jeffrey Chang.  All rights reserved.
+# Revisions Copyright 2007 by Peter Cock.  All rights reserved.
 # This code is part of the Biopython distribution and governed by its
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
@@ -16,6 +13,7 @@ Tested with:
 Release 15.0, July 1998
 Release 16.0, July 1999
 Release 17.0, Dec 2001
+Release 19.0, Mar 2006
 
 
 Classes:
@@ -23,7 +21,6 @@ Record                Holds Prosite data.
 PatternHit            Holds data from a hit against a Prosite pattern.
 Iterator              Iterates over entries in a Prosite file.
 Dictionary            Accesses a Prosite file using a dictionary interface.
-ExPASyDictionary      Accesses Prosite records from ExPASy.
 RecordParser          Parses a Prosite record into a Record object.
 
 _Scanner              Scans Prosite-formatted data.
@@ -38,14 +35,44 @@ _extract_pattern_hits Extract Prosite patterns from a web page.
 
 """
 from types import *
-import string
 import re
 import sgmllib
 from Bio import File
 from Bio import Index
 from Bio.ParserSupport import *
-from Bio.WWW import ExPASy
-from Bio.WWW import RequestLimiter
+
+
+# There is probably a cleaner way to write the read/parse functions
+# if we don't use the "parser = RecordParser(); parser.parse(handle)"
+# approach. Leaving that for the next revision of Bio.Prosite.
+def parse(handle):
+    import cStringIO
+    parser = RecordParser()
+    text = ""
+    for line in handle:
+        text += line
+        if line[:2]=='//':
+            handle = cStringIO.StringIO(text)
+            record = parser.parse(handle)
+            text = ""
+            if not record: # Then this was the copyright notice
+                continue
+            yield record
+
+def read(handle):
+    parser = RecordParser()
+    try:
+        record = parser.parse(handle)
+    except ValueError, error:
+        if error.message=="There doesn't appear to be a record":
+            raise ValueError, "No Prosite record found"
+        else:
+            raise error
+    # We should have reached the end of the record by now
+    remainder = handle.read()
+    if remainder:
+        raise ValueError, "More than one Prosite record found"
+    return record
 
 class Record:
     """Holds information from a Prosite record.
@@ -62,7 +89,8 @@ class Record:
     description    Free-format description.
     pattern        The PROSITE pattern.  See docs.
     matrix         List of strings that describes a matrix entry.
-    rules          List of rule definitions.  (strings)
+    rules          List of rule definitions (from RU lines).  (strings)
+    prorules       List of prorules (from PR lines). (strings)
 
     NUMERICAL RESULTS
     nr_sp_release  SwissProt release.
@@ -84,6 +112,7 @@ class Record:
     cc_author
     cc_ft_key
     cc_ft_desc
+    cc_version     version number (introduced in release 19.0)
 
     DATA BANK REFERENCES - The following are all
                            lists of tuples (swiss-prot accession,
@@ -110,6 +139,8 @@ class Record:
         self.pattern = ''
         self.matrix = []
         self.rules = []
+        self.prorules = []
+        self.postprocessing = []
 
         self.nr_sp_release = ''
         self.nr_sp_seqs = ''
@@ -166,7 +197,7 @@ class PatternHit:
                 lines.append("%7d %10s %s" % (i+1, range_str, seq))
             else:
                 lines.append("%7s %10s %s" % (' ', range_str, seq))
-        return string.join(lines, '\n')
+        return "\n".join(lines)
     
 class Iterator:
     """Returns one record at a time from a Prosite file.
@@ -205,7 +236,7 @@ class Iterator:
                 if line[:2] == '//':
                     break
                 if line[:2] != 'CC':
-                    raise SyntaxError, \
+                    raise ValueError, \
                           "Oops, where's the copyright?"
         
         lines = []
@@ -220,7 +251,7 @@ class Iterator:
         if not lines:
             return None
             
-        data = string.join(lines, '')
+        data = "".join(lines)
         if self._parser is not None:
             return self._parser.parse(File.StringHandle(data))
         return data
@@ -276,6 +307,10 @@ class ExPASyDictionary:
         between each query.
 
         """
+        import warnings
+        from Bio.WWW import RequestLimiter
+        warnings.warn("Bio.Prosite.ExPASyDictionary is deprecated. Please use the function Bio.ExPASy.get_prosite_raw instead.",
+              DeprecationWarning)
         self.parser = parser
         self.limiter = RequestLimiter(delay)
 
@@ -318,6 +353,7 @@ class ExPASyDictionary:
         for the entry.  Raises a KeyError if there's an error.
         
         """
+        from Bio.WWW import ExPASy
         # First, check to see if enough time has passed since my
         # last query.
         self.limiter.wait()
@@ -367,7 +403,8 @@ class _Scanner:
         else:
             uhandle = File.UndoHandle(handle)
 
-        while 1:
+        consumer.finished = False
+        while not consumer.finished:
             line = uhandle.peekline()
             if not line:
                 break
@@ -380,7 +417,7 @@ class _Scanner:
             elif line[:2] == 'CC':
                 self._scan_copyrights(uhandle, consumer)
             else:
-                raise SyntaxError, "There doesn't appear to be a record"
+                raise ValueError, "There doesn't appear to be a record"
 
     def _scan_copyrights(self, uhandle, consumer):
         consumer.start_copyrights()
@@ -453,6 +490,10 @@ class _Scanner:
 ##                else:
 ##                    break
     
+    def _scan_pp(self, uhandle, consumer):
+        #New PP line, PostProcessing, just after the MA line
+        self._scan_line('PP', uhandle, consumer.postprocessing, any_number=1)
+    
     def _scan_ru(self, uhandle, consumer):
         self._scan_line('RU', uhandle, consumer.rule, any_number=1)
     
@@ -470,13 +511,20 @@ class _Scanner:
     def _scan_3d(self, uhandle, consumer):
         self._scan_line('3D', uhandle, consumer.pdb_reference,
                         any_number=1)
-    
+
+    def _scan_pr(self, uhandle, consumer):
+        #New PR line, ProRule, between 3D and DO lines
+        self._scan_line('PR', uhandle, consumer.prorule, any_number=1)
+
     def _scan_do(self, uhandle, consumer):
         self._scan_line('DO', uhandle, consumer.documentation, exactly_one=1)
 
     def _scan_terminator(self, uhandle, consumer):
         self._scan_line('//', uhandle, consumer.terminator, exactly_one=1)
 
+    #This is a list of scan functions in the order expected in the file file.
+    #The function definitions define how many times each line type is exected
+    #(or if optional):
     _scan_fns = [
         _scan_id,
         _scan_ac,
@@ -484,6 +532,7 @@ class _Scanner:
         _scan_de,
         _scan_pa,
         _scan_ma,
+        _scan_pp,
         _scan_ru,
         _scan_nr,
         _scan_cc,
@@ -498,6 +547,7 @@ class _Scanner:
 
         _scan_dr,
         _scan_3d,
+        _scan_pr,
         _scan_do,
         _scan_terminator
         ]
@@ -519,28 +569,28 @@ class _RecordConsumer(AbstractConsumer):
         self._clean_record(self.data)
 
     def identification(self, line):
-        cols = string.split(line)
+        cols = line.split()
         if len(cols) != 3:
-            raise SyntaxError, "I don't understand identification line\n%s" % \
+            raise ValueError, "I don't understand identification line\n%s" % \
                   line
         self.data.name = self._chomp(cols[1])    # don't want ';'
         self.data.type = self._chomp(cols[2])    # don't want '.'
     
     def accession(self, line):
-        cols = string.split(line)
+        cols = line.split()
         if len(cols) != 2:
-            raise SyntaxError, "I don't understand accession line\n%s" % line
+            raise ValueError, "I don't understand accession line\n%s" % line
         self.data.accession = self._chomp(cols[1])
     
     def date(self, line):
-        uprline = string.upper(line)
-        cols = string.split(uprline)
+        uprline = line.upper()
+        cols = uprline.split()
 
         # Release 15.0 contains both 'INFO UPDATE' and 'INF UPDATE'
         if cols[2] != '(CREATED);' or \
            cols[4] != '(DATA' or cols[5] != 'UPDATE);' or \
            cols[7][:4] != '(INF' or cols[8] != 'UPDATE).':
-            raise SyntaxError, "I don't understand date line\n%s" % line
+            raise ValueError, "I don't understand date line\n%s" % line
 
         self.data.created = cols[1]
         self.data.data_update = cols[3]
@@ -554,18 +604,22 @@ class _RecordConsumer(AbstractConsumer):
     
     def matrix(self, line):
         self.data.matrix.append(self._clean(line))
-    
+
+    def postprocessing(self, line):
+        postprocessing = self._clean(line).split(";")
+        self.data.postprocessing.extend(postprocessing)
+
     def rule(self, line):
         self.data.rules.append(self._clean(line))
     
     def numerical_results(self, line):
-        cols = string.split(self._clean(line), ';')
+        cols = self._clean(line).split(";")
         for col in cols:
             if not col:
                 continue
-            qual, data = map(string.lstrip, string.split(col, '='))
+            qual, data = [word.lstrip() for word in col.split("=")]
             if qual == '/RELEASE':
-                release, seqs = string.split(data, ',')
+                release, seqs = data.split(",")
                 self.data.nr_sp_release = release
                 self.data.nr_sp_seqs = int(seqs)
             elif qual == '/FALSE_NEG':
@@ -587,24 +641,32 @@ class _RecordConsumer(AbstractConsumer):
                 elif(qual == "/FALSE_POS"):
                     self.data.nr_false_pos = hits
             else:
-                raise SyntaxError, "Unknown qual %s in comment line\n%s" % \
+                raise ValueError, "Unknown qual %s in comment line\n%s" % \
                       (repr(qual), line)
     
     def comment(self, line):
-        cols = string.split(self._clean(line), ';')
+        #Expect CC lines like this:
+        #CC   /TAXO-RANGE=??EPV; /MAX-REPEAT=2;
+        #Can (normally) split on ";" and then on "="
+        cols = self._clean(line).split(";")
         for col in cols:
-            # DNAJ_2 in Release 15 has a non-standard comment line:
-            # CC   Automatic scaling using reversed database
-            # Throw it away.  (Should I keep it?)
             if not col or col[:17] == 'Automatic scaling':
+                # DNAJ_2 in Release 15 has a non-standard comment line:
+                # CC   Automatic scaling using reversed database
+                # Throw it away.  (Should I keep it?)
                 continue
-            qual, data = map(string.lstrip, string.split(col, '='))
+            if col.count("=") == 0 :
+                #Missing qualifier!  Can we recover gracefully?
+                #For example, from Bug 2403, in PS50293 have:
+                #CC /AUTHOR=K_Hofmann; N_Hulo
+                continue
+            qual, data = [word.lstrip() for word in col.split("=")]
             if qual == '/TAXO-RANGE':
                 self.data.cc_taxo_range = data
             elif qual == '/MAX-REPEAT':
                 self.data.cc_max_repeat = data
             elif qual == '/SITE':
-                pos, desc = string.split(data, ',')
+                pos, desc = data.split(",")
                 self.data.cc_site.append((int(pos), desc))
             elif qual == '/SKIP-FLAG':
                 self.data.cc_skip_flag = data
@@ -618,16 +680,18 @@ class _RecordConsumer(AbstractConsumer):
                 self.data.cc_ft_key = data
             elif qual == '/FT_DESC':
                 self.data.cc_ft_desc = data
+            elif qual == '/VERSION':
+                self.data.cc_version = data
             else:
-                raise SyntaxError, "Unknown qual %s in comment line\n%s" % \
+                raise ValueError, "Unknown qual %s in comment line\n%s" % \
                       (repr(qual), line)
             
     def database_reference(self, line):
-        refs = string.split(self._clean(line), ';')
+        refs = self._clean(line).split(";")
         for ref in refs:
             if not ref:
                 continue
-            acc, name, type = map(string.strip, string.split(ref, ','))
+            acc, name, type = [word.strip() for word in ref.split(",")]
             if type == 'T':
                 self.data.dr_positive.append((acc, name))
             elif type == 'F':
@@ -639,18 +703,23 @@ class _RecordConsumer(AbstractConsumer):
             elif type == '?':
                 self.data.dr_unknown.append((acc, name))
             else:
-                raise SyntaxError, "I don't understand type flag %s" % type
+                raise ValueError, "I don't understand type flag %s" % type
     
     def pdb_reference(self, line):
-        cols = string.split(line)
+        cols = line.split()
         for id in cols[1:]:  # get all but the '3D' col
             self.data.pdb_structs.append(self._chomp(id))
     
+    def prorule(self, line):
+        #Assume that each PR line can contain multiple ";" separated rules
+        rules = self._clean(line).split(";")
+        self.data.prorules.extend(rules)
+
     def documentation(self, line):
         self.data.pdoc = self._chomp(self._clean(line))
 
     def terminator(self, line):
-        pass
+        self.finished = True
 
     def _chomp(self, word, to_chomp='.,;'):
         # Remove the punctuation at the end of a word.
@@ -661,7 +730,7 @@ class _RecordConsumer(AbstractConsumer):
     def _clean(self, line, rstrip=1):
         # Clean up a line.
         if rstrip:
-            return string.rstrip(line[5:])
+            return line[5:].rstrip()
         return line[5:]
 
 def scan_sequence_expasy(seq=None, id=None, exclude_frequent=None):
@@ -675,6 +744,7 @@ def scan_sequence_expasy(seq=None, id=None, exclude_frequent=None):
     will be excluded.
 
     """
+    from Bio import ExPASy
     if (seq and id) or not (seq or id):
         raise ValueError, "Please specify either a sequence or an id"
     handle = ExPASy.scanprosite1(seq, id, exclude_frequent)
@@ -696,7 +766,7 @@ def _extract_pattern_hits(handle):
             self._current_hit = None
             self._last_found = None   # Save state of parsing
         def handle_data(self, data):
-            if string.find(data, 'try again') >= 0:
+            if data.find('try again') >= 0:
                 self.broken_message = data
                 return
             elif data == 'illegal':
@@ -704,14 +774,14 @@ def _extract_pattern_hits(handle):
                 return
             if not self._in_pre:
                 return
-            elif not string.strip(data):
+            elif not data.strip():
                 return
             if self._last_found is None and data[:4] == 'PDOC':
                 self._current_hit.pdoc = data
                 self._last_found = 'pdoc'
             elif self._last_found == 'pdoc':
                 if data[:2] != 'PS':
-                    raise SyntaxError, "Expected accession but got:\n%s" % data
+                    raise ValueError, "Expected accession but got:\n%s" % data
                 self._current_hit.accession = data
                 self._last_found = 'accession'
             elif self._last_found == 'accession':
@@ -811,5 +881,4 @@ def _extract_record(handle):
     p.feed(handle.read())
     if not p.data:
         raise ValueError, "No data found in web page."
-    return string.join(p.data, '')
-
+    return "".join(p.data)
