@@ -7,42 +7,27 @@
 # license.  Please see the LICENSE file that should have been included
 # as part of this package.
 
-import os, sys, getopt, re, time
-from string import maketrans
-from Bio import Fasta
+import re, time
+from Bio import SeqIO
 from Bio import Translate
 from Bio.Seq import Seq
 from Bio import Alphabet
 from Bio.Alphabet import IUPAC
 from Bio.Data import IUPACData, CodonTable
 
+
 ######################################
 # DNA
 ######################
 # {{{ 
 
-_before = ''.join(IUPACData.ambiguous_dna_complement.keys())
-_after = ''.join(IUPACData.ambiguous_dna_complement.values())
-_ttable = maketrans(_before, _after)
-
-def complement(seq):
-    '''returns the complementary sequence (NOT antiparallel)'''
-    #Much faster on really long sequences than the previous loop based one.
-    #thx to Michael Palmer, University of Waterloo
-    return seq.translate(_ttable)
-
 def reverse(seq):
-   " reverse the sequence "
+   """Reverse the sequence. Works on string sequences.
+   """
    r = map(None, seq)
    r.reverse()
    return ''.join(r)
 
-def antiparallel(seq):
-   " returns reversed complementary sequence ( = other strand ) "
-   s = complement(seq)
-   s = reverse(s)
-   return s
-
 def GC(seq):
    """ calculates G+C content """
 #      19/8/03: Iddo: added provision for lowercase
@@ -60,19 +45,19 @@ def GC(seq):
    return gc*100.0/len(seq)
     
 def GC123(seq):
-   " calculates totla G+C content plus first, second and third position "
-   l = len(seq)
+   " calculates total G+C content plus first, second and third position "
 
    d= {}
    for nt in ['A','T','G','C']:
       d[nt] = [0,0,0]
 
-   for i in range(0,l,3):
+   for i in range(0,len(seq),3):
       codon = seq[i:i+3]
       if len(codon) <3: codon += '  '
       for pos in range(0,3):
          for nt in ['A','T','G','C']:
-            if codon[pos] == nt: d[nt][pos] = d[nt][pos] +1
+            if codon[pos] == nt or codon[pos] == nt.lower():
+                d[nt][pos] = d[nt][pos] +1
 
 
    gc = {}
@@ -291,8 +276,9 @@ def six_frame_translations(seq, genetic_code = 1):
    nice looking 6 frame translation with GC content - code from xbbtools
    similar to DNA Striders six-frame translation
    """
-   comp = complement(seq)
-   anti = reverse(comp)
+   from Bio.Seq import reverse_complement
+   anti = reverse_complement(seq)
+   comp = anti[::-1]
    length = len(seq)
    frames = {}
    for i in range(0,3):
@@ -377,13 +363,10 @@ def apply_on_multi_fasta(file, function, *args):
    except:
       raise NotImplementedError, "%s not implemented" % function
    
-   parser = Fasta.RecordParser()
    handle = open(file, 'r')
-   iter = Fasta.Iterator(handle, parser)
+   records = SeqIO.parse(handle, "fasta")
    results = []
-   while 1:
-      record = iter.next()
-      if not record: break
+   for record in records:
       arguments = [record.sequence]
       for arg in args: arguments.append(arg)
       result = f(*arguments)
@@ -416,6 +399,7 @@ def quicker_apply_on_multi_fasta(file, function, *args):
 # {{{ 
 
 if __name__ == '__main__':
+   import sys, getopt
    # crude command line options to use most functions directly on a FASTA file
    options = {'apply_on_multi_fasta':0,
               'quick':0,
