@@ -11,19 +11,21 @@ http://www.expasy.ch/sprot/sprot-top.html
 Tested with:
 Release 37, Release 38, Release 39
 
+Limited testing with:
+Release 51, 54
+
 
 Classes:
 Record             Holds SwissProt data.
 Reference          Holds reference data from a SwissProt entry.
 Iterator           Iterates over entries in a SwissProt file.
 Dictionary         Accesses a SwissProt file using a dictionary interface.
-ExPASyDictionary   Accesses SwissProt records from ExPASy.
 RecordParser       Parses a SwissProt record into a Record object.
-SequenceParser     Parses a SwissProt record into a Seq object.
+SequenceParser     Parses a SwissProt record into a SeqRecord object.
 
 _Scanner           Scans SwissProt-formatted data.
-_RecordConsumer    Consumes SwissProt data to a Record object.
-_SequenceConsumer  Consumes SwissProt data to a Seq object.
+_RecordConsumer    Consumes SwissProt data to a SProt.Record object.
+_SequenceConsumer  Consumes SwissProt data to a SeqRecord object.
 
 
 Functions:
@@ -38,8 +40,8 @@ from Bio import Alphabet
 from Bio import Seq
 from Bio import SeqRecord
 from Bio.ParserSupport import *
-from Bio.WWW import ExPASy
-from Bio.WWW import RequestLimiter
+
+_CHOMP = " \n\r\t.,;" #whitespace and trailing punctuation
 
 class Record:
     """Holds information from a SwissProt record.
@@ -62,6 +64,8 @@ class Record:
     organism_classification  The taxonomy classification.  List of strings.
                              (http://www.ncbi.nlm.nih.gov/Taxonomy/)
     taxonomy_id       A list of NCBI taxonomy id's.
+    host_organism     A list of NCBI taxonomy id's of the hosts of a virus,
+                      if any.
     references        List of Reference objects.
     comments          List of strings.
     cross_references  List of tuples (db, id1[, id2][, id3]).  See the docs.
@@ -91,6 +95,7 @@ class Record:
         self.organelle = ''
         self.organism_classification = []
         self.taxonomy_id = []
+        self.host_organism = []
         self.references = []
         self.comments = []
         self.cross_references = []
@@ -223,6 +228,10 @@ class ExPASyDictionary:
         between each query.
 
         """
+        import warnings
+        from Bio.WWW import RequestLimiter
+        warnings.warn("Bio.SwissProt.ExPASyDictionary is deprecated. Please use the function Bio.ExPASy.get_sprot_raw instead.",
+              DeprecationWarning)
         self.parser = parser
         self.limiter = RequestLimiter(delay)
 
@@ -265,6 +274,7 @@ class ExPASyDictionary:
         for the entry.  Raises a KeyError if there's an error.
         
         """
+        from Bio.WWW import ExPASy
         # First, check to see if enough time has passed since my
         # last query.
         self.limiter.wait()
@@ -291,11 +301,10 @@ class RecordParser(AbstractParser):
         return self._consumer.data
 
 class SequenceParser(AbstractParser):
-    """Parses SwissProt data into a Seq object.
-
+    """Parses SwissProt data into a standard SeqRecord object.
     """
     def __init__(self, alphabet = Alphabet.generic_protein):
-        """Initialize a RecordParser.
+        """Initialize a SequenceParser.
 
         Arguments:
         o alphabet - The alphabet to use for the generated Seq objects. If
@@ -329,12 +338,25 @@ class _Scanner:
         else:
             uhandle = File.UndoHandle(handle)
         
-        while uhandle.peekline():
-            self._scan_record(uhandle, consumer)
+        self._scan_record(uhandle, consumer)
+
+    def _skip_starstar(self, uhandle) :
+        """Ignores any lines starting **"""
+        #See Bug 2353, some files from the EBI have extra lines
+        #starting "**" (two asterisks/stars), usually between the
+        #features and sequence but not all the time.  They appear
+        #to be unofficial automated annotations. e.g.
+        #**
+        #**   #################    INTERNAL SECTION    ##################
+        #**HA SAM; Annotated by PicoHamap 1.88; MF_01138.1; 09-NOV-2003.
+        while "**" == uhandle.peekline()[:2] :
+            skip = uhandle.readline()
+            #print "Skipping line: %s" % skip.rstrip()
 
     def _scan_record(self, uhandle, consumer):
         consumer.start_record()
         for fn in self._scan_fns:
+            self._skip_starstar(uhandle)
             fn(self, uhandle, consumer)
 
             # In Release 38, ID N33_HUMAN has a DR buried within comments.
@@ -378,10 +400,12 @@ class _Scanner:
     def _scan_dt(self, uhandle, consumer):
         self._scan_line('DT', uhandle, consumer.date, exactly_one=1)
         self._scan_line('DT', uhandle, consumer.date, exactly_one=1)
-        self._scan_line('DT', uhandle, consumer.date, exactly_one=1)
+        # IPI doesn't necessarily contain the third line about annotations
+        self._scan_line('DT', uhandle, consumer.date, up_to_one=1)
 
     def _scan_de(self, uhandle, consumer):
-        self._scan_line('DE', uhandle, consumer.description, one_or_more=1)
+        # IPI can be missing a DE line
+        self._scan_line('DE', uhandle, consumer.description, any_number=1)
     
     def _scan_gn(self, uhandle, consumer):
         self._scan_line('GN', uhandle, consumer.gene_name, any_number=1)
@@ -401,6 +425,10 @@ class _Scanner:
         self._scan_line('OX', uhandle, consumer.taxonomy_id,
                         any_number=1)
 
+    def _scan_oh(self, uhandle, consumer):
+        # viral host organism. introduced after SwissProt 39.
+        self._scan_line('OH', uhandle, consumer.organism_host, any_number=1)
+
     def _scan_reference(self, uhandle, consumer):
         while 1:
             if safe_peekline(uhandle)[:2] != 'RN':
@@ -412,6 +440,10 @@ class _Scanner:
             # ws:2001-12-05 added, for record with RL before RA
             self._scan_rl(uhandle, consumer)
             self._scan_ra(uhandle, consumer)
+            #EBI copy of P72010 is missing the RT line, and has one
+            #of their ** lines in its place noting "**   /NO TITLE."
+            #See also bug 2353
+            self._skip_starstar(uhandle) 
             self._scan_rt(uhandle, consumer)
             self._scan_rl(uhandle, consumer)
     
@@ -429,11 +461,21 @@ class _Scanner:
     
     def _scan_rx(self, uhandle, consumer):
         self._scan_line('RX', uhandle, consumer.reference_cross_reference,
-                        up_to_one=1)
+                        any_number=1)
     
     def _scan_ra(self, uhandle, consumer):
+        # In UniProt release 1.12 of 6/21/04, there is a new RG
+        # (Reference Group) line, which references a group instead of
+        # an author.  Each block must have at least 1 RA or RG line.
         self._scan_line('RA', uhandle, consumer.reference_author,
-                        one_or_more=1)
+                        any_number=1)
+        self._scan_line('RG', uhandle, consumer.reference_author,
+                        any_number=1)
+        # PRKN_HUMAN has RG lines, then RA lines.  The best solution
+        # is to write code that accepts either of the line types.
+        # This is the quick solution...
+        self._scan_line('RA', uhandle, consumer.reference_author,
+                        any_number=1)
     
     def _scan_rt(self, uhandle, consumer):
         self._scan_line('RT', uhandle, consumer.reference_title,
@@ -457,7 +499,10 @@ class _Scanner:
     
     def _scan_ft(self, uhandle, consumer):
         self._scan_line('FT', uhandle, consumer.feature_table, any_number=1)
-    
+
+    def _scan_pe(self, uhandle, consumer):
+        self._scan_line('PE', uhandle, consumer.protein_existence, any_number=1)
+
     def _scan_sq(self, uhandle, consumer):
         self._scan_line('SQ', uhandle, consumer.sequence_header, exactly_one=1)
     
@@ -477,9 +522,11 @@ class _Scanner:
         _scan_og,
         _scan_oc,
         _scan_ox,
+        _scan_oh,
         _scan_reference,
         _scan_cc,
         _scan_dr,
+        _scan_pe,
         _scan_kw,
         _scan_ft,
         _scan_sq,
@@ -497,6 +544,9 @@ class _RecordConsumer(AbstractConsumer):
     def __init__(self):
         self.data = None
         
+    def __repr__(self) :
+        return "Bio.SwissProt.SProt._RecordConsumer()"
+        
     def start_record(self):
         self.data = Record()
         
@@ -505,68 +555,154 @@ class _RecordConsumer(AbstractConsumer):
 
     def identification(self, line):
         cols = line.split()
-        self.data.entry_name = cols[1]
-        self.data.data_class = self._chomp(cols[2])    # don't want ';'
-        self.data.molecule_type = self._chomp(cols[3]) # don't want ';'
-        self.data.sequence_length = int(cols[4])
-
+        #Prior to release 51, included with MoleculeType:
+        #ID   EntryName DataClass; MoleculeType; SequenceLength.
+        #
+        #Newer files lack the MoleculeType:
+        #ID   EntryName DataClass; SequenceLength.
+        #
+        #Note that cols is split on white space, so the length
+        #should become two fields (number and units)
+        if len(cols) == 6 :
+            self.data.entry_name = cols[1]
+            self.data.data_class = cols[2].rstrip(_CHOMP)    # don't want ';'
+            self.data.molecule_type = cols[3].rstrip(_CHOMP) # don't want ';'
+            self.data.sequence_length = int(cols[4])
+        elif len(cols) == 5 :
+            self.data.entry_name = cols[1]
+            self.data.data_class = cols[2].rstrip(_CHOMP)    # don't want ';'
+            self.data.molecule_type = None
+            self.data.sequence_length = int(cols[3])
+        else :
+            #Should we print a warning an continue?
+            raise ValueError("ID line has unrecognised format:\n"+line)
+        
         # data class can be 'STANDARD' or 'PRELIMINARY'
         # ws:2001-12-05 added IPI
-        if self.data.data_class not in ['STANDARD', 'PRELIMINARY', 'IPI']: 
-            raise SyntaxError, "Unrecognized data class %s in line\n%s" % \
+        # pjc:2006-11-02 added 'Reviewed' and 'Unreviewed'
+        if self.data.data_class not in ['STANDARD', 'PRELIMINARY', 'IPI',
+                                        'Reviewed', 'Unreviewed']: 
+            raise ValueError, "Unrecognized data class %s in line\n%s" % \
                   (self.data.data_class, line)
         # molecule_type should be 'PRT' for PRoTein
-        if self.data.molecule_type != 'PRT':
-            raise SyntaxError, "Unrecognized molecule type %s in line\n%s" % \
+        # Note that has been removed in recent releases (set to None)
+        if self.data.molecule_type is not None \
+        and self.data.molecule_type != 'PRT':
+            raise ValueError, "Unrecognized molecule type %s in line\n%s" % \
                   (self.data.molecule_type, line)
     
     def accession(self, line):
-        cols = self._chomp(line[5:].rstrip()).split(';')
+        cols = line[5:].rstrip(_CHOMP).strip().split(';')
         for ac in cols:
-            self.data.accessions.append(ac.lstrip())
+            if ac.strip() :
+                #remove any leading or trailing white space
+                self.data.accessions.append(ac.strip())
     
     def date(self, line):
         uprline = string.upper(line)
-        cols = line.split()
-        if uprline.find("CREATED") >= 0:
-            # ws:2001-12-05 prevent e.g. (IPIrel. , created)
-            # !no number given! from crashing
-            if self._chomp(cols[3]) == '':                            #<=
-                self.data.created = cols[1], 0                        #<=
-	    else:	                                              #<=
-                self.data.created = cols[1], int(self._chomp(cols[3]))
-        elif uprline.find('LAST SEQUENCE UPDATE') >= 0:
-            # ws:2001-12-05 prevent e.g. (IPIrel. , created)
-            # !no number given! from crashing
-            if self._chomp(cols[3]) == '':                            #<=
-                self.data.sequence_update = cols[1], 0                #<=
-	    else:                                                     #<=
-                self.data.sequence_update = cols[1], int(self._chomp(cols[3]))
-        elif uprline.find( 'LAST ANNOTATION UPDATE') >= 0:
-            # ws:2001-12-05 prevent e.g. (IPIrel. , created)
-            # !no number given! from crashing
-            if self._chomp(cols[3]) == '':                               #<=
-                self.data.annotation_update = cols[1], 0                 #<=
-	    else:                                                        #<=
-                self.data.annotation_update = cols[1], \
-                                              int(self._chomp(cols[3]))  #<=
+        cols = line.rstrip().split()
+                
+        if uprline.find('CREATED') >= 0 \
+        or uprline.find('LAST SEQUENCE UPDATE') >= 0 \
+        or uprline.find('LAST ANNOTATION UPDATE') >= 0:
+            # Old style DT line
+            # =================
+            # e.g.
+            # DT   01-FEB-1995 (Rel. 31, Created)
+            # DT   01-FEB-1995 (Rel. 31, Last sequence update)
+            # DT   01-OCT-2000 (Rel. 40, Last annotation update)
+            #
+            # or:
+            # DT   08-JAN-2002 (IPI Human rel. 2.3, Created)
+            # ... 
+            
+            # find where the version information will be located
+            # This is needed for when you have cases like IPI where
+            # the release verison is in a different spot:
+            # DT   08-JAN-2002 (IPI Human rel. 2.3, Created)
+            uprcols = uprline.split()
+            rel_index = -1
+            for index in range(len(uprcols)):
+                if uprcols[index].find("REL.") >= 0:
+                    rel_index = index
+            assert rel_index >= 0, \
+                    "Could not find Rel. in DT line: %s" % (line)
+            version_index = rel_index + 1
+            # get the version information
+            str_version = cols[version_index].rstrip(_CHOMP)
+            # no version number
+            if str_version == '':
+                version = 0
+            # dot versioned
+            elif str_version.find(".") >= 0:
+                version = str_version
+            # integer versioned
+            else:
+                version = int(str_version)
+
+            if uprline.find('CREATED') >= 0:
+                self.data.created = cols[1], version
+            elif uprline.find('LAST SEQUENCE UPDATE') >= 0:
+                self.data.sequence_update = cols[1], version
+            elif uprline.find( 'LAST ANNOTATION UPDATE') >= 0:
+                self.data.annotation_update = cols[1], version
+            else:
+                assert False, "Shouldn't reach this line!"
+        elif uprline.find('INTEGRATED INTO') >= 0 \
+        or uprline.find('SEQUENCE VERSION') >= 0 \
+        or uprline.find('ENTRY VERSION') >= 0:
+            # New style DT line
+            # =================
+            # As of UniProt Knowledgebase release 7.0 (including
+            # Swiss-Prot release 49.0 and TrEMBL release 32.0) the
+            # format of the DT lines and the version information
+            # in them was changed - the release number was dropped.
+            #
+            # For more information see bug 1948 and
+            # http://ca.expasy.org/sprot/relnotes/sp_news.html#rel7.0
+            #
+            # e.g.
+            # DT   01-JAN-1998, integrated into UniProtKB/Swiss-Prot.
+            # DT   15-OCT-2001, sequence version 3.
+            # DT   01-APR-2004, entry version 14.
+            #
+            #This is a new style DT line...
+
+            # The date should be in string cols[1]
+            # Get the version number if there is one.
+            # For the three DT lines above: 0, 3, 14
+            try:
+                version = int(cols[-1])
+            except ValueError :
+                version = 0
+
+            # Re-use the historical property names, even though
+            # the meaning has changed slighty:
+            if uprline.find("INTEGRATED") >= 0:
+                self.data.created = cols[1], version
+            elif uprline.find('SEQUENCE VERSION') >= 0:
+                self.data.sequence_update = cols[1], version
+            elif uprline.find( 'ENTRY VERSION') >= 0:
+                self.data.annotation_update = cols[1], version
+            else:
+                assert False, "Shouldn't reach this line!"
         else:
-            raise SyntaxError, "I don't understand the date line %s" % line
+            raise ValueError, "I don't understand the date line %s" % line
     
     def description(self, line):
-        self.data.description = self.data.description + line[5:]
+        self.data.description += line[5:]
     
     def gene_name(self, line):
-        self.data.gene_name = self.data.gene_name + line[5:]
+        self.data.gene_name += line[5:]
     
     def organism_species(self, line):
-        self.data.organism = self.data.organism + line[5:]
+        self.data.organism += line[5:]
     
     def organelle(self, line):
-        self.data.organelle = self.data.organelle + line[5:]
+        self.data.organelle += line[5:]
     
     def organism_classification(self, line):
-        line = self._chomp(line[5:].rstrip())
+        line = line[5:].rstrip(_CHOMP)
         cols = line.split(';')
         for col in cols:
             self.data.organism_classification.append(col.lstrip())
@@ -583,7 +719,7 @@ class _RecordConsumer(AbstractConsumer):
         # To parse this, I need to check to see whether I'm at the
         # first line.  If I am, grab the description and make sure
         # it's an NCBI ID.  Then, grab all the id's.
-        line = self._chomp(line[5:].rstrip())
+        line = line[5:].rstrip(_CHOMP)
         index = line.find('=')
         if index >= 0:
             descr = line[:index]
@@ -592,7 +728,20 @@ class _RecordConsumer(AbstractConsumer):
         else:
             ids = line.split(',')
         self.data.taxonomy_id.extend([id.strip() for id in ids])
-    
+
+    def organism_host(self, line):
+        # Line type OH (Organism Host) for viral hosts
+        # same code as in taxonomy_id()
+        line = line[5:].rstrip(_CHOMP)
+        index = line.find('=')
+        if index >= 0:
+            descr = line[:index]
+            assert descr == "NCBI_TaxID", "Unexpected taxonomy type %s" % descr
+            ids = line[index+1:].split(',')
+        else:
+            ids = line.split(',')
+        self.data.host_organism.extend([id.strip() for id in ids])
+        
     def reference_number(self, line):
         rn = line[5:].rstrip()
         assert rn[0] == '[' and rn[-1] == ']', "Missing brackets %s" % rn
@@ -651,43 +800,40 @@ class _RecordConsumer(AbstractConsumer):
         # reported by edvard@farmasi.uit.no
         # and these can be more complicated like:
         # RX   MEDLINE=95385798; PubMed=7656980;
+        # RX   PubMed=15060122; DOI=10.1136/jmg 2003.012781;
         # We look for these cases first and deal with them
-        if line.find( "=") != -1:
-            cols = line.split()
-            assert len(cols) > 1, "I don't understand RX line %s" % line
-
-            for info_col in cols[1:]:
-                id_cols = info_col.split("=")
-                if len(id_cols) == 2:
-                    self.data.references[-1].references.append(
-                        (self._chomp(id_cols[0]), self._chomp(id_cols[1])))
-                else:
-                    raise AssertionError("I don't understand RX line %s"
-                                         % line)
+        if line.find("=") != -1:
+            cols = line[2:].split("; ")
+            cols = [x.strip() for x in cols]
+            cols = [x for x in cols if x]
+            for col in cols:
+                x = col.split("=")
+                assert len(x) == 2, "I don't understand RX line %s" % line
+                key, value = x[0].rstrip(_CHOMP), x[1].rstrip(_CHOMP)
+                ref = self.data.references[-1].references
+                ref.append((key, value))
         # otherwise we assume we have the type 'RX   MEDLINE; 85132727.'
         else:
             cols = line.split()
             # normally we split into the three parts
-            if len(cols) == 3:
-                self.data.references[-1].references.append(
-                    (self._chomp(cols[1]), self._chomp(cols[2])))
-            else:
-                raise AssertionError("I don't understand RX line %s" % line)
+            assert len(cols) == 3, "I don't understand RX line %s" % line
+            self.data.references[-1].references.append(
+                (cols[1].rstrip(_CHOMP), cols[2].rstrip(_CHOMP)))
     
     def reference_author(self, line):
         assert self.data.references, "RA: missing RN"
         ref = self.data.references[-1]
-        ref.authors = ref.authors + line[5:]
+        ref.authors += line[5:]
     
     def reference_title(self, line):
         assert self.data.references, "RT: missing RN"
         ref = self.data.references[-1]
-        ref.title = ref.title + line[5:]
+        ref.title += line[5:]
     
     def reference_location(self, line):
         assert self.data.references, "RL: missing RN"
         ref = self.data.references[-1]
-        ref.location = ref.location + line[5:]
+        ref.location += line[5:]
     
     def comment(self, line):
         if line[5:8] == '-!-':   # Make a new comment
@@ -697,16 +843,16 @@ class _RecordConsumer(AbstractConsumer):
                 # TCMO_STRGA in Release 37 has comment with no topic
                 self.data.comments.append(line[9:])
             else:
-                self.data.comments[-1] = self.data.comments[-1] + line[9:]
+                self.data.comments[-1] += line[9:]
         elif line[5:8] == '---':
             # If there are no comments, and it's not the closing line,
             # make a new comment.
             if not self.data.comments or self.data.comments[-1][:3] != '---':
                 self.data.comments.append(line[5:])
             else:
-                self.data.comments[-1] = self.data.comments[-1] + line[5:]
+                self.data.comments[-1] += line[5:]
         else:  # copyright notice
-            self.data.comments[-1] = self.data.comments[-1] + line[5:]
+            self.data.comments[-1] += line[5:]
     
     def database_cross_reference(self, line):
         # From CLD1_HUMAN, Release 39:
@@ -718,12 +864,12 @@ class _RecordConsumer(AbstractConsumer):
         i = line.find('[')
         if i >= 0:
             line = line[:i]
-        cols = self._chomp(line.rstrip()).split(';')
+        cols = line.rstrip(_CHOMP).split(';')
         cols = [col.lstrip() for col in cols]
         self.data.cross_references.append(tuple(cols))
     
     def keyword(self, line):
-        cols = self._chomp(line[5:].rstrip()).split(';')
+        cols = line[5:].rstrip(_CHOMP).split(';')
         self.data.keywords.extend([c.lstrip() for c in cols])
 
     def feature_table(self, line):
@@ -783,6 +929,10 @@ class _RecordConsumer(AbstractConsumer):
 
         return description
     
+    def protein_existence(self, line):
+        #TODO - Record this information?
+        pass
+    
     def sequence_header(self, line):
         cols = line.split()
         assert len(cols) == 8, "I don't understand SQ line %s" % line
@@ -796,14 +946,6 @@ class _RecordConsumer(AbstractConsumer):
     def terminator(self, line):
         pass
 
-    # from Python 2.2.2 could be replaced with word.rstrip(".,;")
-    # if there is always only one puctuation
-    def _chomp(self, word, to_chomp='.,;'):
-        # Remove the punctuation at the end of a word.
-        if word[-1] in to_chomp:
-            return word[:-1]
-        return word
-
     #def _clean(self, line, rstrip=1):
     #    if rstrip:
     #        return string.rstrip(line[5:])
@@ -826,13 +968,13 @@ class _RecordConsumer(AbstractConsumer):
             setattr(ref, m, attr.rstrip())
 
 class _SequenceConsumer(AbstractConsumer):
-    """Consumer that converts a SwissProt record to a Seq object.
+    """Consumer that converts a SwissProt record to a SeqRecord object.
 
     Members:
     data      Record with SwissProt data.
     alphabet  The alphabet the generated Seq objects will have.
-
     """
+    #TODO - Cope with references as done for GenBank
     def __init__(self, alphabet = Alphabet.generic_protein):
         """Initialize a Sequence Consumer
 
@@ -847,8 +989,13 @@ class _SequenceConsumer(AbstractConsumer):
         seq = Seq.Seq("", self.alphabet)
         self.data = SeqRecord.SeqRecord(seq)
         self.data.description = ""
+        self.data.name = ""
+        self._current_ref = None
         
     def end_record(self):
+        if self._current_ref is not None:
+            self.data.annotations['references'].append(self._current_ref)
+            self._current_ref = None
         self.data.description = self.data.description.rstrip()
 
     def identification(self, line):
@@ -856,8 +1003,20 @@ class _SequenceConsumer(AbstractConsumer):
         self.data.name = cols[1]
 
     def accession(self, line):
-        ids = line[5:].rstrip().split(';')
-        self.data.id = ids[0]
+        #Note that files can and often do contain multiple AC lines.
+        ids = line[5:].strip().split(';')
+        #Remove any white space
+        ids = [x.strip() for x in ids if x.strip()]
+        
+        #Use the first as the ID, but record them ALL in the annotations
+        try :
+            self.data.annotations['accessions'].extend(ids)
+        except KeyError :
+            self.data.annotations['accessions'] = ids
+            
+        #Use the FIRST accession as the ID, not the first on this line!
+        self.data.id = self.data.annotations['accessions'][0]
+        #self.data.id = ids[0]
 
     def description(self, line):
         self.data.description = self.data.description + \
@@ -868,6 +1027,223 @@ class _SequenceConsumer(AbstractConsumer):
                       self.alphabet)
         self.data.seq = self.data.seq + seq
 
+    def gene_name(self, line):
+        #We already store the identification/accession as the records name/id
+        try :
+            self.data.annotations['gene_name'] += line[5:]
+        except KeyError :
+            self.data.annotations['gene_name'] =  line[5:]
+
+    def comment(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores the comments as a long string with newlines
+        #with key 'comment'
+        try :
+            self.data.annotations['comment'] += "\n" + line[5:]
+        except KeyError :
+            self.data.annotations['comment'] =  line[5:]
+        #TODO - Follow SwissProt conventions more closely?
+
+    def database_cross_reference(self, line):
+        #Format of the line is described in the manual dated 04-Dec-2007 as:
+        #DR   DATABASE; PRIMARY; SECONDARY[; TERTIARY][; QUATERNARY].
+        #However, some older files only seem to have a single identifier:
+        #DR   DATABASE; PRIMARY.
+        #
+        #Also must cope with things like this from Tests/SwissProt/sp007,
+        #DR   PRODOM [Domain structure / List of seq. sharing at least 1 domain]
+        #
+        #Store these in the dbxref list, but for consistency with
+        #the GenBank parser and with what BioSQL can cope with,
+        #store only DATABASE_IDENTIFIER:PRIMARY_IDENTIFIER
+        parts = [x.strip() for x in line[5:].strip(_CHOMP).split(";")]
+        if len(parts) > 1 :
+            value = "%s:%s" % (parts[0], parts[1])
+            #Avoid duplicate entries
+            if value not in self.data.dbxrefs :
+                self.data.dbxrefs.append(value)
+        #else :
+            #print "Bad DR line:\n%s" % line
+            
+                    
+    def date(self, line):
+        date_str = line.split()[0]
+        uprline = string.upper(line)
+        if uprline.find('CREATED') >= 0 :
+            #Try and agree with SeqRecord convention from the GenBank parser,
+            #which stores the submitted date as 'date'
+            self.data.annotations['date'] = date_str
+        elif uprline.find('LAST SEQUENCE UPDATE') >= 0 :
+            #There is no existing convention from the GenBank SeqRecord parser
+            self.data.annotations['date_last_sequence_update'] = date_str
+        elif uprline.find('LAST ANNOTATION UPDATE') >= 0:
+            #There is no existing convention from the GenBank SeqRecord parser
+            self.data.annotations['date_last_annotation_update'] = date_str
+
+    def keyword(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores a list as 'keywords'
+        cols = line[5:].rstrip(_CHOMP).split(';')
+        cols = [c.strip() for c in cols]
+        cols = filter(None, cols)
+        try :
+            #Extend any existing list of keywords
+            self.data.annotations['keywords'].extend(cols)
+        except KeyError :
+            #Create the list of keywords
+            self.data.annotations['keywords'] = cols
+
+    def organism_species(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores the organism as a string with key 'organism'
+        data = line[5:].rstrip(_CHOMP)
+        try :
+            #Append to any existing data split over multiple lines
+            self.data.annotations['organism'] += " " + data
+        except KeyError:
+            self.data.annotations['organism'] = data
+
+    def organism_host(self, line):
+        #There is no SeqRecord convention from the GenBank parser,
+        data = line[5:].rstrip(_CHOMP)
+        index = data.find('=')
+        if index >= 0:
+            descr = data[:index]
+            assert descr == "NCBI_TaxID", "Unexpected taxonomy type %s" % descr
+            ids = data[index+1:].split(',')
+        else:
+            ids = data.split(',')
+
+        try :
+            #Append to any existing data
+            self.data.annotations['organism_host'].extend(ids)
+        except KeyError:
+            self.data.annotations['organism_host'] = ids
+
+    def organism_classification(self, line):
+        #Try and agree with SeqRecord convention from the GenBank parser,
+        #which stores this taxonomy lineage ese as a list of strings with
+        #key 'taxonomy'.
+        #Note that 'ncbi_taxid' is used for the taxonomy ID (line OX)
+        line = line[5:].rstrip(_CHOMP)
+        cols = [col.strip() for col in line.split(';')]
+        try :
+            #Append to any existing data
+            self.data.annotations['taxonomy'].extend(cols)
+        except KeyError:
+            self.data.annotations['taxonomy'] = cols
+
+    def taxonomy_id(self, line):
+        #Try and agree with SeqRecord convention expected in BioSQL
+        #the NCBI taxon id with key 'ncbi_taxid'.
+        #Note that 'taxonomy' is used for the taxonomy lineage
+        #(held as a list of strings, line type OC)
+
+        line = line[5:].rstrip(_CHOMP)
+        index = line.find('=')
+        if index >= 0:
+            descr = line[:index]
+            assert descr == "NCBI_TaxID", "Unexpected taxonomy type %s" % descr
+            ids = line[index+1:].split(',')
+        else:
+            ids = line.split(',')
+
+        try :
+            #Append to any existing data
+            self.data.annotations['ncbi_taxid'].extend(ids)
+        except KeyError:
+            self.data.annotations['ncbi_taxid'] = ids
+
+    def reference_number(self, line):
+        """RN line, reference number (start of new reference)."""
+        from Bio.SeqFeature import Reference
+        # if we have a current reference that hasn't been added to
+        # the list of references, add it.
+        if self._current_ref is not None:
+            self.data.annotations['references'].append(self._current_ref)
+        else:
+            self.data.annotations['references'] = []
+
+        self._current_ref = Reference()
+
+    def reference_position(self, line):
+        """RP line, reference position."""
+        assert self._current_ref is not None, "RP: missing RN"
+        #Should try and store this in self._current_ref.location
+        #but the SwissProt locations don't match easily to the
+        #format used in GenBank...
+        pass
+
+    def reference_cross_reference(self, line):
+        """RX line, reference cross-references."""
+        assert self._current_ref is not None, "RX: missing RN"
+        # The basic (older?) RX line is of the form:
+        # RX   MEDLINE; 85132727.
+        # or more recently:
+        # RX   MEDLINE=95385798; PubMed=7656980;
+        # RX   PubMed=15060122; DOI=10.1136/jmg 2003.012781;
+        # We look for these cases first and deal with them
+        if line.find("=") != -1:
+            cols = line[2:].split("; ")
+            cols = [x.strip() for x in cols]
+            cols = [x for x in cols if x]
+            for col in cols:
+                x = col.split("=")
+                assert len(x) == 2, "I don't understand RX line %s" % line
+                key, value = x[0].rstrip(_CHOMP), x[1].rstrip(_CHOMP)
+                if key == "MEDLINE" :
+                    self._current_ref.medline_id = value
+                elif key == "PubMed" :
+                    self._current_ref.pubmed_id = value
+                else :
+                    #Sadly the SeqFeature.Reference object doesn't
+                    #support anything else (yet)
+                    pass
+        # otherwise we assume we have the type 'RX   MEDLINE; 85132727.'
+        else:
+            # CLD1_HUMAN in Release 39 and DADR_DIDMA in Release 33
+            # have extraneous information in the RX line.  Check for
+            # this and chop it out of the line.
+            # (noticed by katel@worldpath.net)
+            ind = line.find('[NCBI, ExPASy, Israel, Japan]')
+            if ind >= 0:
+                line = line[:ind]
+            cols = line.split()
+            # normally we split into the three parts
+            assert len(cols) == 3, "I don't understand RX line %s" % line
+            key = cols[1].rstrip(_CHOMP)
+            value = cols[2].rstrip(_CHOMP)
+            if key == "MEDLINE" :
+                self._current_ref.medline_id = value
+            elif key == "PubMed" :
+                self._current_ref.pubmed_id = value
+            else :
+                #Sadly the SeqFeature.Reference object doesn't
+                #support anything else (yet)
+                pass
+
+    def reference_author(self, line):
+        """RA line, reference author(s)."""
+        assert self._current_ref is not None, "RA: missing RN"
+        self._current_ref.authors += line[5:].rstrip("\n")
+
+    def reference_title(self, line):
+        """RT line, reference title."""
+        assert self._current_ref is not None, "RT: missing RN"
+        self._current_ref.title += line[5:].rstrip("\n")
+    
+    def reference_location(self, line):
+        """RL line, reference 'location' - journal, volume, pages, year."""
+        assert self._current_ref is not None, "RL: missing RN"
+        self._current_ref.journal += line[5:].rstrip("\n")
+
+    def reference_comment(self, line):
+        """RC line, reference comment."""
+        assert self._current_ref is not None, "RC: missing RN"
+        #This has a key=value; structure...
+        #Can we do a better job with the current Reference class?
+        self._current_ref.comment += line[5:].rstrip("\n")
+
 def index_file(filename, indexname, rec2key=None):
     """index_file(filename, indexname, rec2key=None)
 
@@ -903,3 +1279,32 @@ def index_file(filename, indexname, rec2key=None):
             raise KeyError, "duplicate key %s found" % key
 
         index[key] = start, length
+        
+if __name__ == "__main__" :
+    print "Quick self test..."
+
+    example_filename = "../../Tests/SwissProt/sp008"
+
+    import os
+    if not os.path.isfile(example_filename) :
+        print "Missing test file %s" % example_filename
+    else :
+        #Try parsing it!
+        
+        handle = open(example_filename)
+        for record in Iterator(handle, RecordParser()) :
+            print record.entry_name
+            print ",".join(record.accessions)
+            print record.keywords
+            print repr(record.organism)
+            print record.sequence[:20] + "..."
+        handle.close()
+
+        handle = open(example_filename)
+        for record in Iterator(handle, SequenceParser()) :
+            print record.name
+            print record.id
+            print record.annotations['keywords']
+            print repr(record.annotations['organism'])
+            print record.seq.tostring()[:20] + "..."
+        handle.close()
