@@ -63,6 +63,9 @@ class DBObject(RegisterableObject):
             x = _support.make_rate_limited_function(self._get, delay)
             setattr(self, "_get", x)
         if timeout is not None:
+            import warnings
+            warnings.warn("Using timeouts has been deprecated, as this code relies on Bio.MultiProc, which itself has been deprecated. If you need this functionality, please let the Biopython developers know by sending an email to biopython-dev@biopython.org.",
+              DeprecationWarning)
             x = _support.make_timed_function(
                 self._get, timeout,
                 self._make_pickleable, self._unmake_pickleable)
@@ -140,6 +143,10 @@ class DBGroup(RegisterableGroup):
         RegisterableGroup.__init__(self, name, abbrev, doc)
         if behavior not in ['concurrent', 'serial']:
             raise ValueError, "behavior must be 'concurrent' or 'serial'"
+        if behavior=='concurrent':
+            import warnings
+            warnings.warn("Concurrent behavior has been deprecated, as this functionality needs Bio.MultiProc, which itself has been deprecated. If you need the concurrent behavior, please let the Biopython developers know by sending an email to biopython-dev@biopython.org to avoid permanent removal of this feature.",
+              DeprecationWarning)
         self.behavior = behavior
         self._last_object_used = None
 
@@ -226,7 +233,7 @@ class TextLikeMixin:
     def _check_for_errors(self, handle, failure_cases):
         from Martel import Parser
         from Bio import StdHandler
-        from Bio.ReseekFile import ReseekFile
+        from Bio.EUtils.ReseekFile import ReseekFile
         
         if not failure_cases:
             return handle
