@@ -1,3 +1,8 @@
+import warnings
+warnings.warn("Bio.expressions was deprecated, as it does not work with recent versions of mxTextTools. If you want to continue to use this module, please get in contact with the Biopython developers at biopython-dev@biopython.org to avoid permanent removal of this module from Biopython", DeprecationWarning)
+
+
+
 from Martel import *
 from Bio import Std
 
@@ -311,7 +316,7 @@ ungapped_lambda_stats = (
       ToEol() )
 )
 gapped_lambda_stats = (
-    Str("Gapped") + AnyEol() + 
+    Opt(Str("Gapped") + AnyEol()) + 
     Str("Lambda     K      H") + AnyEol() + 
     ( Spaces() + float_stat("gapped_lambda") + 
       Spaces() + float_stat("gapped_k") + 
