@@ -20,7 +20,7 @@ def test_mean_median(module):
   elif module=='Pycluster':
     from Pycluster import mean, median
   else:
-    raise 'Unknown module name', module
+    raise ValueError('Unknown module name: ' + module)
   print "test_mean_median:"
   data1 = array([ 34.3, 3, 2 ])
   data2 = [ 5, 10 ,15, 20]
@@ -39,7 +39,7 @@ def test_matrix_parse(module):
   elif module=='Pycluster':
     from Pycluster import treecluster
   else:
-    raise 'Unknown module name', module
+    raise ValueError('Unknown module name: ' + module)
   print "test_matrix_parse:"
   # Normal matrix, no errors
   data1 = array([[ 1.1, 1.2 ],
@@ -133,7 +133,7 @@ def test_kcluster(module):
   elif module=='Pycluster':
     from Pycluster import kcluster
   else:
-    raise 'Unknown module name', module
+    raise ValueError('Unknown module name: ' + module)
   print "test_kcluster"
   nclusters = 3
   # First data set
@@ -178,9 +178,8 @@ def test_kcluster(module):
 
   # test first data set
   print "First data set"
-  clusterid, centroids, error, nfound = kcluster (data1, nclusters=nclusters, mask=mask1, weight=weight1, transpose=0, npass=100, method='a', dist='e')
+  clusterid, error, nfound = kcluster (data1, nclusters=nclusters, mask=mask1, weight=weight1, transpose=0, npass=100, method='a', dist='e')
   print "Number of cluster ids is %d (should be %d)" % (len(clusterid), len(data1))
-  print "Number of centroids is %d (should be %d)" % (len(centroids), nclusters)
   correct = [0,1,1,2]
   mapping = [clusterid[correct.index(i)] for i in range(nclusters)]
   same = 1
@@ -188,18 +187,11 @@ def test_kcluster(module):
     if clusterid[i]!=mapping[correct[i]]: same = 0
   if same: print "Correct clustering solution found."
   else: print "Wrong clustering solution found."
-  print "Cluster centroids:"
-  for i in range(nclusters):
-    centroid = centroids[mapping[i]]
-    for number in centroid:
-      print "%7.3f " % number,
-    print
 
   # test second data set
   print "Second data set"
-  clusterid, centroids, error, nfound = kcluster (data2, nclusters=3, mask=mask2, weight=weight2, transpose=0, npass=100, method='a', dist='e')
+  clusterid, error, nfound = kcluster (data2, nclusters=3, mask=mask2, weight=weight2, transpose=0, npass=100, method='a', dist='e')
   print "Number of cluster ids is %d (should be %d)" % (len(clusterid), len(data2))
-  print "Number of centroids is %d (should be %d)" % (len(centroids), nclusters)
   correct = [0, 0, 0, 0, 0, 0, 1, 1, 2, 1, 1, 1, 1]
   mapping = [clusterid[correct.index(i)] for i in range(nclusters)]
   same = 1
@@ -207,12 +199,6 @@ def test_kcluster(module):
     if clusterid[i]!=mapping[correct[i]]: same = 0
   if same: print "Correct clustering solution found."
   else: print "Wrong clustering solution found."
-  print "Cluster centroids:"
-  for i in range(nclusters):
-    centroid = centroids[mapping[i]]
-    for number in centroid:
-      print "%7.3f " % number,
-    print
   print
 
 def test_clusterdistance(module):
@@ -221,7 +207,7 @@ def test_clusterdistance(module):
   elif module=='Pycluster':
     from Pycluster import clusterdistance
   else:
-    raise 'Unknown module name', module
+    raise ValueError('Unknown module name: ' + module)
   print "test_clusterdistance:"
 
   # First data set
@@ -299,7 +285,7 @@ def test_treecluster(module):
   elif module=='Pycluster':
     from Pycluster import treecluster
   else:
-    raise 'Unknown module name', module
+    raise ValueError('Unknown module name: ' + module)
   print "test_treecluster:"
   # First data set
   weight1 =  [ 1,1,1,1,1 ]
@@ -345,62 +331,78 @@ def test_treecluster(module):
   print "First data set:"
   print_matrix(data1, mask1)
   print "Pairwise average-linkage clustering"
-  result, linkdist = treecluster(data=data1, mask=mask1, weight=weight1, applyscale=0, transpose=0, method='a', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data1)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data1)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='a', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data1)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
 
   print "Pairwise single-linkage clustering"
-  result, linkdist = treecluster(data=data1, mask=mask1, weight=weight1, applyscale=0, transpose=0, method='s', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data1)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data1)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='s', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data1)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
 
   print "Pairwise centroid-linkage clustering"
-  result, linkdist = treecluster(data=data1, mask=mask1, weight=weight1, applyscale=0, transpose=0, method='c', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data1)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data1)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='c', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data1)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
 
   print "Pairwise maximum-linkage clustering"
-  result, linkdist = treecluster(data=data1, mask=mask1, weight=weight1, applyscale=0, transpose=0, method='m', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data1)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data1)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data1, mask=mask1, weight=weight1, transpose=0, method='m', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data1)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
 
   # Test second data set
   print "Second data set:"
   print "Pairwise average-linkage clustering"
-  result, linkdist = treecluster(data=data2, mask=mask2, weight=weight2, applyscale=0, transpose=0, method='a', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data2)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data2)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='a', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data2)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
 
   print "Pairwise single-linkage clustering"
-  result, linkdist = treecluster(data=data2, mask=mask2, weight=weight2, applyscale=0, transpose=0, method='s', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data2)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data2)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='s', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data2)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
 
   print "Pairwise centroid-linkage clustering"
-  result, linkdist = treecluster(data=data2, mask=mask2, weight=weight2, applyscale=0, transpose=0, method='c', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data2)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data2)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='c', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data2)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
 
   print "Pairwise maximum-linkage clustering"
-  result, linkdist = treecluster(data=data2, mask=mask2, weight=weight2, applyscale=0, transpose=0, method='m', dist='e')
-  print "Number of nodes is %d (should be %d)" % (len(result), len(data2)-1)
-  print "Number of link distances is %d (should be %d)" % (len(linkdist), len(data2)-1)
-  for i in range(len(result)):
-    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, result[i][0], result[i][1], linkdist[i])
+  tree = treecluster(data=data2, mask=mask2, weight=weight2, transpose=0, method='m', dist='e')
+  print "Number of nodes is %d (should be %d)" % (len(tree), len(data2)-1)
+  for i in range(len(tree)):
+    left = tree[i].left
+    right = tree[i].right
+    distance = tree[i].distance
+    print "Node %3d joins node %3d with node %3d; link distance is %7.3f" % (i, left, right, distance)
   print
 
 def test_somcluster(module):
@@ -409,7 +411,7 @@ def test_somcluster(module):
   elif module=='Pycluster':
     from Pycluster import somcluster
   else:
-    raise 'Unknown module name', module
+    raise ValueError('Unknown module name: ' + module)
   print "test_somcluster:"
 
   # First data set
@@ -463,6 +465,42 @@ def test_somcluster(module):
   print "Grid is %d-dimensional (should be 2-dimensional)" % len(clusterid[0])
   print
 
+def test_distancematrix_kmedoids(module):
+  if module=='Bio.Cluster':
+    from Bio.Cluster import distancematrix, kmedoids
+  elif module=='Pycluster':
+    from Pycluster import distancematrix, kmedoids
+  else:
+    raise ValueError('Unknown module name: ' + module)
+
+  print "test_distancematrix_kmedoids:"
+  data = array([[2.2, 3.3, 4.4],
+                [2.1, 1.4, 5.6],
+                [7.8, 9.0, 1.2],
+                [4.5, 2.3, 1.5],
+                [4.2, 2.4, 1.9],
+                [3.6, 3.1, 9.3],
+                [2.3, 1.2, 3.9],
+                [4.2, 9.6, 9.3],
+                [1.7, 8.9, 1.1]])
+  mask = array([[1, 1, 1],
+                [1, 1, 1],
+                [0, 1, 1],
+                [1, 1, 1],
+                [1, 1, 1],
+                [0, 1, 0],
+                [1, 1, 1],
+                [1, 0, 1],
+                [1, 1, 1]])
+  weight = array([2.0, 1.0, 0.5])
+  matrix = distancematrix(data, mask=mask, weight=weight)
+  print "Distance matrix:"
+  for row in matrix:
+      print " ".join(["%7.3f" % number for number in row])
+  clusterid, error, nfound = kmedoids(matrix, npass=1000)
+  print "Cluster assignments:", clusterid
+  print "Within-cluster sum of distances: %7.3f" % error
+
 def run_tests(module="Pycluster"):
   if module==[]: module = "Bio.Cluster"
   test_mean_median(module)
@@ -471,3 +509,9 @@ def run_tests(module="Pycluster"):
   test_clusterdistance(module)
   test_treecluster(module)
   test_somcluster(module)
+  test_distancematrix_kmedoids(module)
+
+if __name__ == "__main__" :
+  print "test_Cluster"
+  run_tests(module = "Bio.Cluster")
+
