@@ -15,24 +15,19 @@ from Bio.GenBank import utils
 
 gb_file_dir = os.path.join(os.getcwd(), 'GenBank')
 
-#test_files = ['cor6_6.gb', 'arab1.gb', 'iro.gb', 'arab2.gb', 'pri1.gb',
-#              'arab3.gb', 'arab4.gb', 'pri2.gb', 'bct1.gb', 'bct2.gb']
-
 test_files = ['noref.gb', 'cor6_6.gb', 'iro.gb', 'pri1.gb', 'arab1.gb',
-              'protein_refseq.gb', 'extra_keywords.gb']
-test_files += ['one_of.gb', 'NT_019265.gb', 'origin_line.gb', 'blank_seq.gb']
-test_files += ['dbsource_wrap.gb']
+              'protein_refseq.gb', 'extra_keywords.gb', 'one_of.gb',
+              'NT_019265.gb', 'origin_line.gb', 'blank_seq.gb',
+              'dbsource_wrap.gb', 'gbvrl1_start.seq']
 
-write_format_files = test_files[:]
+# We only test writing on a subset of the examples:
+write_format_files = ['noref.gb', 'cor6_6.gb', 'iro.gb', 'pri1.gb', 'arab1.gb',
+                      'extra_keywords.gb', 'one_of.gb', 'origin_line.gb']
 # don't test writing on protein_refseq, since it is horribly nasty
 # don't test writing on the CONTIG refseq, because the wrapping of
 # locations won't work exactly
 # don't test writing on blank_seq because it lacks a sequence type
 # don't test dbsource_wrap because it is a junky RefSeq file
-for remove_file in ["protein_refseq.gb", "NT_019265.gb", "blank_seq.gb",
-                    "dbsource_wrap.gb"]:
-    if remove_file in write_format_files:
-        write_format_files.remove(remove_file)
 
 files_to_parse = []
 for file in test_files:
@@ -53,8 +48,12 @@ record_parser = GenBank.RecordParser(debug_level = 0)
 all_parsers = [feature_parser, record_parser]
 print "Testing parsers..."
 for parser in all_parsers:
-    for file in files_to_parse:
-        handle = open(file, 'r')
+    for filename in files_to_parse:
+        if not os.path.isfile(filename) :
+            print "Missing test input file: %s" % filename
+            continue
+        
+        handle = open(filename, 'r')
         iterator = GenBank.Iterator(handle, parser)
         
         while 1:
@@ -65,7 +64,7 @@ for parser in all_parsers:
 
             if isinstance(parser, GenBank.FeatureParser):
                 print "***Record from the FeatureParser"
-                print "Seq:", cur_record.seq
+                print "Seq:", repr(cur_record.seq)
                 print "Id:", cur_record.id
                 print "Name:", cur_record.name
                 print "Description", cur_record.description
@@ -101,40 +100,9 @@ for parser in all_parsers:
                          
         handle.close()
         
-# test GenBank dictionary
-def remove_index(indexname):
-    """Remove the index if it exists.
-    """
-    if os.path.exists(indexname):
-        try:
-            os.remove(indexname) # remove files -- old Biopython
-        except OSError: # is a directory -- new
-            for filename in os.listdir(indexname):
-                os.remove(os.path.join(indexname, filename))
-            os.removedirs(indexname)
-
-print "Testing dictionaries..."
-dict_file = os.path.join(gb_file_dir, 'cor6_6.gb')
-indexname = os.path.join(gb_file_dir, 'cor6_6.idx')
-
-print "Indexing file to serve as a dictionary..."
-remove_index(indexname)
-GenBank.index_file(dict_file, indexname)
-gb_dict = GenBank.Dictionary(indexname, GenBank.FeatureParser())
-
-print "len:", len(gb_dict)
-k = gb_dict.keys()
-k.sort()
-print "keys:", k
-
-# pick out some keys and make sure we are getting back decent records
-for key in k[:3]:
-    print "Retrieving record with key %s" % key
-    cur_seqrecord = gb_dict[key]
-    print "description:", cur_seqrecord.description
-    print "id:", cur_seqrecord.id
-
-remove_index(indexname)
+#The dictionaries code has been deprecated
+#print "Testing dictionaries..."
+#...
 
 # test writing GenBank format
 print "Testing writing GenBank format..."
