@@ -60,13 +60,13 @@ class DesTests(unittest.TestCase):
 
         try:
             rec = p.parse(corruptRec)
-            assert 0, "Should never get here"
-        except SyntaxError, e :
+            assert False, "Should never get here"
+        except ValueError, e :
             pass
 
     def testRecord(self) :
         recLine = '49268\tsp\tb.1.2.1\t-\tHuman (Homo sapiens)    \n'
-        recFields = ('49268','sp','b.1.2.1','','Human (Homo sapiens)')
+        recFields = (49268,'sp','b.1.2.1','','Human (Homo sapiens)')
 
         rec = Des.Parser().parse(recLine)
         assert rec.sunid == recFields[0]
